<?php
/**
 * Midas_Campaigns:: defines an API for campaign storage backends for Midas.
 *
 * See the enclosed file COPYING for license information (GPL). If you did not
 * receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * $Horde: midas/lib/Campaigns.php,v 1.22 2004/09/09 04:27:23 chuck Exp $
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @version $Revision: 1.22 $
 * @since   Midas 0.1
 * @package Midas
 */
class Midas_Campaigns {

    var $_params;
    var $_vfs;
    var $_clients;

    function Midas_Campaigns($params)
    {
        $this->_params = $params;
    }

    /**
     * Display an ad.
     *
     * @public
     *
     * @param string $driver    What display driver to use for the ad.
     * @param array  $criteria  Any criteria, such as specific ad ID to display
     *                          or certain characteristics to use to select an
     *                          ad.
     *
     * @return string  An ad to display.
     */
    function showAd($driver, &$criteria)
    {
        /* Get the display object based on the driver requested. */
        require_once MIDAS_BASE . '/lib/Display.php';
        $display = Midas_Display::singleton($driver);

        /* If no specific ad requested, go through the filters specified in
         * $criteria to select the right ad to display. */
        if (empty($criteria['ad_id'])) {
            $campaigns = $this->getCampaignList('active');
            $this->filterCampaigns($campaigns, $criteria);

            /* Now get a random ad from the filtered campaign list. */
            $criteria['ad_id'] = $this->getRandomAdId($campaigns);
        }

        $ad = $this->getCampaign($criteria['ad_id']);
        if (is_a($ad, 'PEAR_Error')) {
            return sprintf(_("Could not display ad. %s"), $ad->getMessage());
        }

        return $display->show($ad);
    }

    /**
     * Returns a random campaign from an array of campaigns. Also tries to
     * make sure that no campaign is shown twice on the same page.
     *
     * @public
     *
     * @param array $campaigns  An array of campaigns to choose from.
     *
     * @return int  A random campaign id.
     */
    function getRandomAdId($campaigns)
    {
        if (!count($campaigns)) {
            /* No list of campaigns to select from so get all from backend. */
            $campaigns = $this->getCampaignList('active');

            /* Check if still empty, in which case return an error message. */
            if (empty($campaigns)) {
                return PEAR::raiseError(_("No ads available"));
            }
        }
        $random_campaign = array_rand($campaigns);
        $ad_id = $campaigns[$random_campaign]['campaign_id'];
        /* Check if there is a session record of the currently viewed ad. This
         * is to prevent showing the same ad several times on the same page.
         * This is done by making sure that no ad is shown again in the same 30
         * seconds time span. If there is no other ad to choose from, then it
         * is shown. */
        $time = time();
        if (isset($_SESSION['midas'][$ad_id])) {
            /* This ad has been viewed previously. */
            $orig_ad_id = $ad_id;
            $already_shown = ($time - $_SESSION['midas'][$ad_id]) < 30;

            /* Loop through possible ads until an ad that has not been shown in
             * the last 30secs, but only a maximum of 5 tries or until the
             * possible choices are exhausted. */
            $tries = 0;
            while (count($campaigns) && $already_shown && $tries < 5) {
                $random_campaign = array_rand($campaigns);
                $ad_id = $campaigns[$random_campaign]['campaign_id'];
                /* If ad has not been registered as recently shown, set
                 * $already_shown to false, breaking out of the while loop. */
                if (($already_shown = isset($_SESSION['midas'][$ad_id]))) {
                    $already_shown = ($time - $_SESSION['midas'][$ad_id]) < 30;
                }
                unset($campaigns[$random_campaign]);
                $tries++;
            }

            /* If all tries exhausted and still the selected ad was already
             * shown recently just go for the ad initially selected. */
            if ($already_shown) {
                $ad_id = $orig_ad_id;
            }
        }

        /* Store the time of the view of the current ad. */
        $_SESSION['midas'][$ad_id] = $time;

        return $ad_id;
    }

    /**
     * Returns the url for an ad.
     *
     * @public
     *
     * @param int $ad_id  The ID of an ad for which to fetch the URL.
     *
     * @return string  The URL.
     */
    function getAdUrl($ad_id)
    {
        $campaign = $this->_getCampaign($ad_id);
        return $campaign['campaign_link'];
    }

    /**
     * Filter campaigns according to specific criteria.
     *
     * @public
     *
     * @param array $campaigns  An array of available campaigns.
     * @param array $criteria   Criteria to be used for filtering the potential
     *                          ads that can be shown.
     */
    function filterCampaigns(&$campaigns, $criteria)
    {
        foreach ($criteria as $key => $val) {
            if (empty($criteria[$key]) && $criteria[$key] != '0') {
                continue;
            }
            $filter = '_filterCampaigns_' . $key;
            $campaigns = call_user_func(array($this, $filter), $campaigns, $val);
        }
    }

    /**
     * Filters campaigns based on keywords.
     *
     * @private
     *
     * @param array $campaigns     An array of available campaigns.
     * @param array $target_words  Words that are to be searched in the
     *                             keywords of the ad.
     *
     * @return array  Campaigns with desired keywords.
     */
    function _filterCampaigns_keywords(&$campaigns, $target_words)
    {
        if (!is_array($target_words)) {
            /* Make sure punctuation is treated the same as word separators. */
            $target_words = str_replace(array('.', ',', ':', ';', '-', '?', '!'), ' ', $target_words);
            /* Split up all the keywords into an array consisting only of
             * unique keywords. */
            $target_words = array_unique(preg_split('|\s+|', $target_words));
        }

        foreach ($campaigns as $key => $campaign) {
            $keywords = explode(',', $campaign['campaign_keywords']);
            $match = array_intersect($target_words, $keywords);
            if (!count($match)) {
                /* No target words in this campaign, remove from list. */
                unset($campaigns[$key]);
            }
        }
        return $campaigns;
    }

    /**
     * Filters campaigns based on type.
     *
     * @private
     *
     * @param array $campaigns  An array of available campaigns.
     * @param array $type       The type of ad to show, can be one of:
     *                            text  - text based ad
     *                            image - ad showing an image
     *
     * @return array  Campaigns of desired type.
     */
    function _filterCampaigns_type(&$campaigns, $type)
    {
        foreach ($campaigns as $key => $campaign) {
            if ($campaigns[$key]['campaign_type'] != $type) {
                /* Campaign does not match requested type, remove from list. */
                unset($campaigns[$key]);
            }
        }
echo "<pre>";var_dump($campaigns);exit;
        return $campaigns;
    }

    /**
     * Filters campaigns based on width. This by default filters also for
     * image files.
     *
     * @private
     *
     * @param array $campaigns  An array of available campaigns.
     * @param array $width      Maximum width of image ad.
     *
     * @return array  Campaigns with desired width.
     */
    function _filterCampaigns_width(&$campaigns, $width)
    {
        foreach ($campaigns as $key => $campaign) {
            /* If not an image or if image has width set and is too large,
             * remove from list. */
            if (!isset($campaigns[$key]['image_width']) ||
                $campaigns[$key]['image_width'] > $width) {
                unset($campaigns[$key]);
            }
        }
        return $campaigns;
    }

    /**
     * Filters campaigns based on height. This by default filters also for
     * image files.
     *
     * @private
     *
     * @param array $campaigns  An array of available campaigns.
     * @param array $height     Maximum height of image ad.
     *
     * @return array  Campaigns with desired height.
     */
    function _filterCampaigns_height(&$campaigns, $height)
    {
        foreach ($campaigns as $key => $campaign) {
            /* If not an image or if image has height set and is too large,
             * remove from list. */
            if (!isset($campaigns[$key]['image_height']) ||
                $campaigns[$key]['image_height'] > $height) {
                unset($campaigns[$key]);
            }
        }
        return $campaigns;
    }

    /**
     * Save the ad to the backend.
     *
     * @public
     *
     * @param array $info  An array of data for the ad to save.
     *
     * @return mixed  True on success or PEAR_Error on failure.
     */
    function saveCampaign(&$info)
    {
        /* An image has been uploaded. */
        if ($info['campaign_type'] == '0') {
            if ($info['campaign_ad']['uploaded'] === true) {
                /* New file uploaded, get the temp file storing the image. */
                $tmp_file = $info['campaign_ad']['file'];

                /* Get a name for the image to be used in VFS. */
                $id = md5(mt_rand()) . '_' . $info['campaign_ad']['name'];

                /* Save to VFS. */
                $save = $GLOBALS['vfs']->write('.horde/midas', $id, $tmp_file, true);
                if (is_a($save, 'PEAR_Error')) {
                    return $save;
                }
                $info['campaign_ad'] = $id;
            } else {
                /* No new file uploaded, store the image from tmp directory. */
                $file_name = basename($info['campaign_ad']['file']);
                $GLOBALS['vfs']->write('.horde/midas', $file_name, $info['campaign_ad']['file'], true);
                $info['campaign_ad'] = $file_name;
            }
        }

        /* Save campaign info to backend and return. */
        return $this->_saveCampaign($info);
    }

    /**
     * Returns ad information from the backend.
     *
     * @public
     *
     * @param int $campaign_id  The ID of the ad to return.
     *
     * @return array  Campaign information as an array.
     */
    function &getCampaign($campaign_id)
    {
        $campaign = $this->_getCampaign($campaign_id);
        if (is_null($campaign)) {
            Horde::logMessage($campaign, __FILE__, __LINE__, PEAR_LOG_ERR);
            return PEAR::raiseError(_("Campaign not found."));
        }
        return $campaign;
    }

    /**
     * Delete a campaign from the backend.
     *
     * @public
     *
     * @param int $campaign_id  The ID of the campaign to delete.
     *
     * @return mixed  True on success or PEAR_Error on failure.
     */
    function deleteCampaign($campaign_id)
    {
        $campaign = $this->_getCampaign($campaign_id);
        if (is_a($campaign, 'PEAR_Error')) {
            return $campaign;
        }

        /* First delete the image file in the VFS if needed. */
        if ($campaign['campaign_type'] == '0') {
            $delete = $GLOBALS['vfs']->deleteFile('.horde/midas', $campaign['campaign_ad']);
            if (is_a($delete, 'PEAR_Error')) {
                return $delete;
            }
        }

        /* Delete the campaign from the backend. */
        return $this->_deleteCampaign($campaign_id);
    }

    /**
     * Get campaign stats.
     *
     * @access public
     *
     * @param optional string $filter  The type of filter to apply to the
     *                                 recordset.
     *
     * @return array  Array of rows and field values, PEAR_Error on failure.
     */
    function getCampaignStats($filter = '')
    {
        return $this->_getCampaignStats($filter);
    }

    /**
     * Get a list of campaigns. This function exists so that client
     * names could be fetched from a separate source (e.g. Turba) and
     * still merged with the campaign list to display the full client
     * name.
     *
     * @access public
     *
     * @param optional string $filter   The type of filter to apply to the
     *                                  recordset.
     *
     * @param optional bool $clients    Fetch also full client names.
     *
     * @return array  Array of rows and field values, PEAR_Error on failure.
     */
    function &getCampaignList($filter = '', $clients = true)
    {
        $campaign_list = $this->_getCampaignList($filter);
        if (is_a($campaign_list, 'PEAR_Error')) {
            return PEAR::raiseError(sprintf(_("Could not fetch a list of campaigns. %s"), $campaign_list->message));
        }

        /* Full client names requested, fetch the list of clients and merge. */
        if ($clients) {
            $client_list = $GLOBALS['clients']->getClients();
            foreach ($campaign_list as $key => $campaign) {
                $campaign_list[$key]['client_name'] = $client_list[$campaign['client_id']];
            }
        }

        return $campaign_list;
    }

    /**
     * Get a list of campaigns for a select box.
     *
     * @access public
     *
     * @param optional string $filter   The type of filter to apply to the
     *                                  recordset.
     *
     * @return array  Array of rows and field values, PEAR_Error on failure.
     */
    function &getCampaignSelect($filter = '')
    {
        $campaign_list = $this->_getCampaignSelect($filter);
        if (is_a($campaign_list, 'PEAR_Error')) {
            return PEAR::raiseError(sprintf(_("Could not fetch a list of campaigns. %s"), $campaign_list->message));
        }

        return $campaign_list;
    }

    /**
     * Attempts to return a concrete Midas_Driver instance based on $driver.
     *
     * @param string $driver          The type of concrete Midas_Driver subclass
     *                                to return. The is based on the storage
     *                                driver ($driver). The code is dynamically
     *                                included.
     *
     * @param optional array $params  A hash containing any additional
     *                                configuration or connection parameters a
     *                                subclass might need.
     *
     * @return mixed   The newly created concrete Midas_Driver instance, or
     *                 false on an error.
     */
    function &factory($driver = null, $params = null)
    {
        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['storage']['driver'];
        }

        $driver = basename($driver);

        if (is_null($params)) {
            $params = Horde::getDriverConfig('storage', $driver);
        }

        require_once dirname(__FILE__) . '/Campaigns/' . $driver . '.php';
        $class = 'Midas_Campaigns_' . $driver;
        if (class_exists($class)) {
            return $ret = &new $class($params);
        } else {
            Horde::fatal(PEAR::raiseError(sprintf(_("No such backend '%s' found"), $driver)), __FILE__, __LINE__);
        }
    }

    /**
     * Attempts to return a reference to a concrete Midas_Driver instance
     * based on $driver.  It wil only create a new instance if no
     * Midas_Driver instance with the same parameters currently exists.
     *
     * This should be used if multiple storage sources are required.
     *
     * This method must be invoked as: $var = &Midas_Driver::singleton()
     *
     * @param string    $driver     The type of concrete Midas_Driver subclass
     *                              to return.  The is based on the storage
     *                              driver ($driver).  The code is dynamically
     *                              included.
     *
     * @param array     $params     (optional) A hash containing any additional
     *                              configuration or connection parameters a
     *                              subclass might need.
     *
     * @return mixed    The created concrete Midas_Driver instance, or false
     *                  on error.
     */
    function &singleton($driver = null, $params = null)
    {
        static $instances;

        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['storage']['driver'];
        }

        if (is_null($params)) {
            $params = Horde::getDriverConfig('storage', $driver);
        }

        if (!isset($instances)) {
            $instances = array();
        }

        $signature = serialize(array($driver, $params));
        if (!isset($instances[$signature])) {
            $instances[$signature] = &Midas_Campaigns::factory($driver, $params);
        }

        return $instances[$signature];
    }

}
