<?php
/**
 * Midas_Clients:: defines an API for client storage backends for Midas.
 *
 * See the enclosed file COPYING for license information (GPL). If you did not
 * receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * $Horde: midas/lib/Clients.php,v 1.12 2003/12/23 04:39:46 chuck Exp $
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @version $Revision: 1.12 $
 * @since   Midas 0.1
 * @package Midas
 */
class Midas_Clients {

    var $_params;

    function Midas_Clients ($params)
    {
        $this->_params = $params;
    }

    function saveClient(&$info)
    {
        /* Save client info to backend and return. */
        return $this->_saveClient($info);
    }

    function deleteClient($client_id)
    {
        $client = $this->_getClient($client_id);
        if (is_a($client, 'PEAR_Error')) {
            return $client;
        }

        $campaigns = $GLOBALS['campaigns']->getClientCampaigns[$client_id];
        foreach ($campaigns as $campaign) {
            $GLOBALS['campaigns']->deleteCampaign($campaign['campaign_id']);
        }

        return true;
    }

    function &getClient($client_id)
    {
        $client = $this->_getClient($client_id);
        if (is_null($client)) {
            Horde::logMessage($client, __FILE__, __LINE__, PEAR_LOG_DEBUG);
            return PEAR::raiseError(_("Client not found."));
        }
        return $client;

    }

    function &getClients($one_col = true)
    {
        $clients = $this->_getClients($one_col);
        if (is_a($clients, 'PEAR_Error')) {
            return array();
        }

        /* TODO: Sorting routines. */
        return $clients;
    }

    function &getClientsSummary()
    {
        /* Get the list of clients. */
        $clients = $this->_getClients(false);
        if (is_a($clients, 'PEAR_Error')) {
            /* Errors are already logged in the backend. */
            return array();
        }

        /* Loop through clients and set up the campaign stats. */
        foreach ($clients as $id => $client) {
            $clients[$id]['client_id'] = $client['client_id'];

            /* Get client's active campaigns. */
            $campaigns = $GLOBALS['campaigns']->numClientCampaigns($client['client_id'], 'active');
            $clients[$id]['campaign_active'] = $campaigns;

            /* Get client's new campaigns. */
            $campaigns = $GLOBALS['campaigns']->numClientCampaigns($client['client_id'], 'new');
            $clients[$id]['campaign_new'] = $campaigns;

            /* Get client's old campaigns. */
            $campaigns = $GLOBALS['campaigns']->numClientCampaigns($client['client_id'], 'old');
            $clients[$id]['campaign_old'] = $campaigns;
        }

        return $clients;
    }

    /**
     * Attempts to return a concrete Midas_Driver instance based on $driver.
     *
     * @param string $driver          The type of concrete Midas_Driver subclass
     *                                to return. The is based on the storage
     *                                driver ($driver). The code is dynamically
     *                                included.
     *
     * @param optional array $params  A hash containing any additional
     *                                configuration or connection parameters a
     *                                subclass might need.
     *
     * @return mixed   The newly created concrete Midas_Driver instance, or
     *                 false on an error.
     */
    function &factory($driver = null, $params = null)
    {
        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['storage']['driver'];
        }

        $driver = basename($driver);

        if (is_null($params)) {
            $params = Horde::getDriverConfig('storage', $driver);
        }

        require_once dirname(__FILE__) . '/Clients/' . $driver . '.php';
        $class = 'Midas_Clients_' . $driver;
        if (class_exists($class)) {
            return $ret = &new $class($params);
        } else {
            Horde::fatal(PEAR::raiseError(sprintf(_("No such backend '%s' found"), $driver)), __FILE__, __LINE__);
        }
    }

    /**
     * Attempts to return a reference to a concrete Midas_Driver instance
     * based on $driver.  It wil only create a new instance if no
     * Midas_Driver instance with the same parameters currently exists.
     *
     * This should be used if multiple storage sources are required.
     *
     * This method must be invoked as: $var = &Midas_Driver::singleton()
     *
     * @param string    $driver     The type of concrete Midas_Driver subclass
     *                              to return.  The is based on the storage
     *                              driver ($driver).  The code is dynamically
     *                              included.
     *
     * @param array     $params     (optional) A hash containing any additional
     *                              configuration or connection parameters a
     *                              subclass might need.
     *
     * @return mixed    The created concrete Midas_Driver instance, or false
     *                  on error.
     */
    function &singleton($driver = null, $params = null)
    {
        static $instances;

        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['storage']['driver'];
        }

        if (is_null($params)) {
            $params = Horde::getDriverConfig('storage', $driver);
        }

        if (!isset($instances)) {
            $instances = array();
        }

        $signature = serialize(array($driver, $params));
        if (!isset($instances[$signature])) {
            $instances[$signature] = &Midas_Clients::factory($driver, $params);
        }

        return $instances[$signature];
    }

}
